#ifndef CONTROL_LOOP_H
#define CONTROL_LOOP_H

#include <QObject>
#include <QList>
#include "defines.h"

class BatteryInfo;
class GridMeter;
class Multi;
class MultiPhaseData;
class PVPowerLimiter;
class Settings;
class SystemCalc;

class ControlLoop : public QObject
{
	Q_OBJECT
public:
	Multi *multi() const
	{
		return mMulti;
	}

	Settings *settings() const
	{
		return mSettings;
	}

	BatteryInfo *batteryInfo() const
	{
		return mBatteryInfo;
	}

	void setBatteryInfo(BatteryInfo *i);

	void setPVPowerLimiter(PVPowerLimiter *l);

	double getGridPower(Phase phase) const;

	GridMeter *acSensor() const
	{
		return mGridMeter;
	}

protected:
	ControlLoop(SystemCalc *systemCalc, Multi *multi, GridMeter *gridMeter, Settings *settings,
				QObject *parent = 0);

	void adjustSetpoint(double gridPower, Phase targetPhase, MultiPhaseData *target,
						double setpoint);

	double adjustTotalSetpoint(double setpoint);

	bool hasSetpoint(Phase phase) const;

	double pvPowerSetpointOffset() const;

private:
	void checkStep();

	void performStep();

	double computeSetpoint(double pTarget) const;

	SystemCalc *mSystemCalc;
	GridMeter *mGridMeter;
	Multi *mMulti;
	Settings *mSettings;
	BatteryInfo *mBatteryInfo;
	PVPowerLimiter *mPVPowerLimiter;
};

#endif // CONTROL_LOOP_H
