#ifndef DBUS_CGWACS_H
#define DBUS_CGWACS_H

#include <QObject>
#include <QList>
#include <QStringList>

class BatteryInfo;
class BatteryLife;
class ControlLoop;
class DbusServiceMonitor;
class Hub4ControlBridge;
class Multi;
class PVPowerLimiter;
class Settings;
class SystemCalc;
class VBusItemGridMeter;
class VBusItemPVInverter;
class VeQItem;

/*!
 * Main object which ties everything together.
 * This class will coordinate the search for new AC sensors, and start
 * acquisition when found. The class `AcSensorUpdater` is responsible for
 * communication with the sensors. Sensor data is stored in `AcSensor`.
 *
 * The class will also make sure that the Hub-4 control loop is started when
 * apropriate. The control loop itself is implemented in `ControlLoop`.
 *
 * Finally, this class will detect the presence of a multi within the setup,
 * which we need for the Hub-4 control loop.
 */
class Hub4Control : public QObject
{
	Q_OBJECT
public:
	Hub4Control(QObject *parent = 0);

private slots:
	void onSerialEvent(const char *description);

	void onServiceAdded(VeQItem *item);

	void onServiceRemoved(VeQItem *item);

	void onTimeZoneChanged();

	void updateControlLoop();

private:
	void setupGridMeter(VeQItem *root);

	QStringList mGridMeterServices;
	DbusServiceMonitor *mServiceMonitor;
	Settings *mSettings;
	VBusItemGridMeter *mGridMeter;
	Multi *mMulti;
	SystemCalc *mSystemCalc;
	BatteryLife *mBatteryLife;
	ControlLoop *mControlLoop;
	BatteryInfo *mBatteryInfo;
	PVPowerLimiter *mPVPowerLimiter;
	Hub4ControlBridge *mHub4ControlBridge;
	VeQItem *mTimeZone;
	VeQItem *mHub4Mode;
};

#endif // DBUS_CGWACS_H
