#include <cmath>
#include "battery_info.h"
#include "battery_life.h"
#include "hub4_control_bridge.h"
#include "settings.h"

Hub4ControlBridge::Hub4ControlBridge(BatteryInfo *batteryInfo, Settings *settings, QObject *parent):
	DBusBridge("pub/com.victronenergy.hub4", true, parent)
{
	Q_ASSERT(settings != 0);
	produce(settings, "acPowerSetPoint", "AcPowerSetpoint", "W", 0);
	produce(settings, "maxChargePercentage", "MaxChargePercentage", "%", 0);
	produce(settings, "maxDischargePercentage", "MaxDischargePercentage", "%", 0);
	produce(settings, "state", "State");
	produce(batteryInfo, "maxChargePower", "MaxChargePower", "W", 0);
	produce(batteryInfo, "maxDischargePower", "MaxDischargePower", "W", 0);
	produce("DeviceInstance", 0);
	registerService();
}

bool Hub4ControlBridge::toDBus(const QString &path, QVariant &value)
{
	if (path == "State") {
		value = static_cast<int>(value.value<BatteryLifeState>());
		return true;
	}
	if (value.type() == QVariant::Double && !qIsFinite(value.toDouble()))
		value = QVariant();
	return true;
}

bool Hub4ControlBridge::fromDBus(const QString &path, QVariant &value)
{
	Q_UNUSED(value)
	if (path == "State") {
		// Item was created to be able to change the battery life state via the hub4 D-Bus service,
		// but it never worked properly. So now we keep it as a read only value.
		// value = QVariant::fromValue(static_cast<BatteryLifeState>(value.toInt()));
		return false;
	} else if (path == "DeviceInstance") {
		return false;
	}
	return true;
}

QString Hub4ControlBridge::toText(const QString &path, const QVariant &value, const QString &unit,
								  int precision)
{
	if (path == "State")
		return BatteryLife::getStateName(static_cast<BatteryLifeState>(value.toInt()));
	return DBusBridge::toText(path, value, unit, precision);
}
