#include <QsLog.h>
#include <velib/qt/ve_qitem.hpp>
#include "dbus_service_monitor.h"
#include "hub4_control.h"
#include "hub4_control_starter.h"

Hub4ControlStarter::Hub4ControlStarter(QObject *parent) :
	QObject(parent),
	mServiceMonitor(new DbusServiceMonitor(this))
{
	connect(mServiceMonitor, SIGNAL(serviceAdded(VeQItem *)),
			this, SLOT(onServiceAdded(VeQItem *)));
}

void Hub4ControlStarter::start()
{
	QLOG_INFO() << "Waiting for Hub-4 assistant";
	mServiceMonitor->start();
}

void Hub4ControlStarter::onServiceAdded(VeQItem *item)
{
	QString serviceName = item->id();
	if (serviceName.startsWith("com.victronenergy.vebus.")) {
		VeQItem *setpoint = item->itemGetOrCreate("Hub4/AcPowerSetpoint");
		connect(setpoint, SIGNAL(valueChanged(VeQItem *, QVariant)),
				this, SLOT(onAcPowerSetpointChanged(VeQItem *, QVariant)));
		onAcPowerSetpointChanged(item, setpoint->getValue());
	}
}

void Hub4ControlStarter::onAcPowerSetpointChanged(VeQItem *item, QVariant value)
{
	Q_UNUSED(item)
	if (!value.isValid())
		return;
	new Hub4Control(parent());
	deleteLater();
}
