#ifndef PVPOWERLIMITER_H
#define PVPOWERLIMITER_H

#include <QObject>

class GridMeter;
class Multi;
class PVInverter;
class Settings;
class SystemCalc;
class QTimer;

class PVPowerLimiter : public QObject
{
	Q_OBJECT
public:
	PVPowerLimiter(SystemCalc *systemCalc, Multi *multi, Settings *settings, QObject *parent = 0);

	void addPVInverter(PVInverter *pvInverter);

	void removePVInverter(PVInverter *pvInverter);

	void setGridMeter(GridMeter *gridMeter);

	double pvPowerSetpointOffset() const;

private slots:
	void updateLimit();

	void onPvInverterProbeTimer();

private:
	enum PvPowerLimitState {
		On,
		Off,
		Probe
	};

	void setPvPowerLimitState(PvPowerLimitState s);

	GridMeter *mGridMeter;
	SystemCalc *mSystemCalc;
	Multi *mMulti;
	Settings *mSettings;
	QList<PVInverter *> mPVInverters;
	QTimer *mPvInverterProbeTimer;

	PvPowerLimitState mState;
	double mPvLimit;
	double mPrevError;
	double mAvgError;
	int mAvgErrorCount;
	double mAvgPower;
	double mAvgConsumption;
	int mProbeCount;
};

#endif // PVPOWERLIMITER_H
