#include <QsLog.h>
#include <QTimer>
#include "grid_meter.h"
#include "battery_info.h"
#include "defines.h"
#include "multi.h"
#include "multi_phase_data.h"
#include "settings.h"
#include "single_phase_control.h"

SinglePhaseControl::SinglePhaseControl(SystemCalc *systemCalc, Multi *multi, GridMeter *gridMeter,
									   Settings *settings, Phase phase, Hub4Mode hub4Mode,
									   QObject *parent):
	ControlLoop(systemCalc, multi, gridMeter, settings, parent),
	mTimer(new QTimer(this)),
	mPhase(phase),
	mMultiTargetPhase(multi->getPhaseData(phase)),
	mHub4Mode(hub4Mode),
	mMultiUpdate(false),
	mMeterUpdate(false),
	mPrevSetpoint(0)
{
	Q_ASSERT(multi != 0);
	Q_ASSERT(multi->getPhaseData(phase) != 0);
	Q_ASSERT(settings != 0);
	mTimer->setInterval(5000);
	mTimer->start();
	connect(mMultiTargetPhase, SIGNAL(acPowerInChanged()),
			this, SLOT(onPowerFromMulti()));
	if (gridMeter != 0) {
		connect(gridMeter, SIGNAL(powerChanged(Phase)),
				this, SLOT(onPowerFromMeter(Phase)));
	}
	connect(mTimer, SIGNAL(timeout()), this, SLOT(onTimer()));
}

void SinglePhaseControl::onTimer()
{
	QLOG_DEBUG() << "Update timeout";
	mMultiUpdate = true;
	mMeterUpdate = true;
	checkStep();
}

void SinglePhaseControl::onPowerFromMulti()
{
	mMultiUpdate = true;
	checkStep();
}

void SinglePhaseControl::onPowerFromMeter(Phase phase)
{
	if (phase != (mHub4Mode == Hub4PhaseCompensation ? MultiPhase : mPhase))
		return;
	mMeterUpdate = true;
	checkStep();
}

void SinglePhaseControl::checkStep()
{
	if (!mMeterUpdate)
		return;
	if (!mMeterUpdate && acSensor() != 0)
		return;
	performStep();
	mMeterUpdate = false;
	mMultiUpdate = false;
	mTimer->start();
}

void SinglePhaseControl::performStep()
{
	double pNet = getGridPower(mHub4Mode == Hub4PhaseCompensation ? MultiPhase : mPhase);
	double gridSetpoint = settings()->acPowerSetPoint();
	double setpoint = mMultiTargetPhase->acPowerIn() + gridSetpoint - pNet + pvPowerSetpointOffset();
	/// @todo EV adjustTotalSetpoint has side effects
	setpoint = adjustTotalSetpoint(setpoint);
	adjustSetpoint(pNet, mPhase, mMultiTargetPhase, setpoint);
}
