#ifndef SINGLE_PHASE_CONTROL_H
#define SINGLE_PHASE_CONTROL_H

#include "control_loop.h"
#include "defines.h"
#include "settings.h"

class GridMeter;
class Multi;
class MultiPhaseData;
class AcSensorPhase;
class QTimer;

/*!
 * Implements the Hub-4 control loop.
 */
class SinglePhaseControl : public ControlLoop
{
	Q_OBJECT
public:
	SinglePhaseControl(SystemCalc *systemCalc, Multi *multi, GridMeter *acSensor, Settings *settings,
					   Phase phase, Hub4Mode hub4Mode, QObject *parent = 0);

private slots:
	void onTimer();

	void onPowerFromMulti();

	void onPowerFromMeter(Phase phase);

private:
	void checkStep();

	void performStep();

	double computeSetpoint() const;

	QTimer *mTimer;
	Phase mPhase;
	MultiPhaseData *mMultiTargetPhase;
	Hub4Mode mHub4Mode;
	bool mMultiUpdate;
	bool mMeterUpdate;

	double mPrevSetpoint;
};

#endif // SINGLE_PHASE_CONTROL_H
