#include "multi.h"
#include "multi_phase_data.h"
#include "settings.h"
#include "split_phase_control.h"

SplitPhaseControl::SplitPhaseControl(SystemCalc *systemCalc, Multi *multi, GridMeter *gridMeter,
									 Settings *settings, QObject *parent):
	MultiPhaseControl(systemCalc, multi, gridMeter, settings, parent),
	mSetpoints(3)
{
}

void SplitPhaseControl::performStep()
{
	Multi *multi = this->multi();
	Settings *settings = this->settings();

	double extraPhasePower = settings->acPowerSetPoint() + pvPowerSetpointOffset();
	int phaseCount = multi->getSetpointCount();
	if (phaseCount == 0)
		return;
	extraPhasePower /= phaseCount;
	for (int p=0; p<3; ++p) {
		Phase phase = static_cast<Phase>(PhaseL1 + p);
		MultiPhaseData *mpd = multi->getPhaseData(phase);
		double pNet = getGridPower(phase);
		mSetpoints[p] = mpd->acPowerIn() + extraPhasePower - pNet;
	}
	adjustSetpoints(mSetpoints);
}
