#include <velib/qt/ve_qitem.hpp>
#include "vbus_item_gridmeter.h"

VBusItemGridMeter::VBusItemGridMeter(VeQItem *service, VeQItem *settings, QObject *parent) :
	GridMeter(parent),
	mIsMultiPhase(false),
	mService(service),
	mSettings(settings),
	mL1Power(createItem(service, "Ac/L1/Power", SLOT(onL1PowerChanged()))),
	mL2Power(createItem(service, "Ac/L2/Power", SLOT(onL2PowerChanged()))),
	mL3Power(createItem(service, "Ac/L3/Power", SLOT(onL3PowerChanged()))),
	mTotalPower(createItem(service, "Ac/Power", SLOT(onTotalPowerChanged())))
{
	Q_ASSERT(service);
	updateIsMultiPhase();
}

bool VBusItemGridMeter::isMultiPhase() const
{
	return mIsMultiPhase;
}

double VBusItemGridMeter::getTotalPower() const
{
	return getDouble(mTotalPower);
}

double VBusItemGridMeter::getPower(Phase p) const
{
	switch (p) {
	case PhaseL1:
		return getDouble(mL1Power);
	case PhaseL2:
		return getDouble(mL2Power);
	case PhaseL3:
		return getDouble(mL3Power);
	case MultiPhase:
		return getDouble(mTotalPower);
	default:
		Q_ASSERT(false);
		return qQNaN();
	}
}

void VBusItemGridMeter::onL1PowerChanged()
{
	updateIsMultiPhase();
	emit powerChanged(PhaseL1);
}

void VBusItemGridMeter::onL2PowerChanged()
{
	updateIsMultiPhase();
	emit powerChanged(PhaseL2);
}

void VBusItemGridMeter::onL3PowerChanged()
{
	updateIsMultiPhase();
	emit powerChanged(PhaseL3);
}

void VBusItemGridMeter::onTotalPowerChanged()
{
	emit powerChanged(MultiPhase);
}

void VBusItemGridMeter::updateIsMultiPhase()
{
	bool isMultiphase = mL2Power->getValue().isValid() || mL3Power->getValue().isValid();
	if (mIsMultiPhase == isMultiphase)
		return;
	mIsMultiPhase = isMultiphase;
	emit isMultiPhaseChanged();
}

VeQItem *VBusItemGridMeter::createItem(VeQItem *root, const QString &path, const char *slot)
{
	VeQItem *result = root->itemGetOrCreate(path);
	connect(result, SIGNAL(valueChanged(VeQItem *, QVariant)), this, slot);
	result->getValue();
	return result;
}

double VBusItemGridMeter::getDouble(VeQItem *item)
{
	bool ok = false;
	QVariant v = item->getValue();
	double d = v.toDouble(&ok);
	return ok ? d : qQNaN();
}
