#include <velib/qt/ve_qitems_dbus.hpp>
#include "vbus_item_pvinverter.h"

VBusItemPVInverter::VBusItemPVInverter(VeQItem *service, QObject *parent):
	PVInverter(parent),
	mRoot(service),
	mPower(createItem(service, "Ac/Power", this, SIGNAL(powerChanged()))),
	mPowerLimit(createItem(service, "Ac/PowerLimit", this, SIGNAL(powerLimitChanged()))),
	mPowerLimitStepSize(createItem(service, "Ac/PowerLimitStepSize", this, SIGNAL(powerLimitStepSizeChanged()))),
	mMinPowerLimit(createItem(service, "Ac/MinPowerLimit", this, SIGNAL(minPowerLimitChanged()))),
	mMaxPower(createItem(service, "Ac/MaxPower", this, SIGNAL(maxPowerChanged())))
{
}

double VBusItemPVInverter::power() const
{
	return getDouble(mPower);
}

double VBusItemPVInverter::powerLimit() const
{
	return getDouble(mPowerLimit);
}

void VBusItemPVInverter::setPowerLimit(double p)
{
	mPowerLimit->setValue(p);
}

double VBusItemPVInverter::powerLimitStepSize() const
{
	return getDouble(mPowerLimitStepSize);
}

double VBusItemPVInverter::minPowerLimit() const
{
	return getDouble(mMinPowerLimit);
}

double VBusItemPVInverter::maxPower() const
{
	return getDouble(mMaxPower);
}

// static
VeQItem *VBusItemPVInverter::createItem(VeQItem *service, const QString &path, QObject *src,
										const char *signal)
{
	VeQItem *result = service->itemGetOrCreate(path);
	connect(result, SIGNAL(valueChanged(VeQItem *, QVariant)), src, signal);
	result->getValue();
	return result;
}

double VBusItemPVInverter::getDouble(VeQItem *item)
{
	QVariant v = item->getValue();
	return v.isValid() ? v.toDouble() : qQNaN();
}
